#
#           py-google-searchCopyright (C) 2018Javinator9889
# This program comes with ABSOLUTELY NO WARRANTY; for details add the-h option.
#      This is free software and you are welcome to redistribute it
#              under certain conditions; type -L for details.
#
from abc import ABCMeta as __ABCMeta


class AvailableColors(metaclass=__ABCMeta):
    Red = "ic:specific,isc:red"
    Orange = "ic:specific,isc:orange"
    Yellow = "ic:specific,isc:yellow"
    Green = "ic:specific,isc:green"
    Teal = "ic:specific,isc:teel"
    Blue = "ic:specific,isc:blue"
    Purple = "ic:specific,isc:purple"
    Pink = "ic:specific,isc:pink"
    White = "ic:specific,isc:white"
    Gray = "ic:specific,isc:gray"
    Black = "ic:specific,isc:black"
    Brown = "ic:specific,isc:brown"
    
    
class AvailableColorsType(metaclass=__ABCMeta):
    Full_color = "ic:color"
    Black_and_white = "ic:gray"
    Transparent = "ic:trans"
    

class AvailableRights(metaclass=__ABCMeta):
    Labeled_for_reuse_with_modifications = "sur:fmc"
    Labeled_for_reuse = "sur:fc"
    Labeled_for_noncommercial_reuse_with_modification = "sur:fm"
    Labeled_for_noncommercial_reuse = "sur:f"


class AvailableSizes(metaclass=__ABCMeta):
    Large = "isz:l"
    Medium = "isz:m"
    Icon = "isz:i"
    BiggerThan400x300 = "isz:lt,islt:qsvga"
    BiggerThan640x480 = "isz:lt,islt:vga"
    BiggerThan800x600 = "isz:lt,islt:svga"
    BiggerThan1024x768 = "visz:lt,islt:xga"
    BiggerThan2MP = "isz:lt,islt:2mp"
    BiggerThan4MP = "isz:lt,islt:4mp"
    BiggerThan6MP = "isz:lt,islt:6mp"
    BiggerThan8MP = "isz:lt,islt:8mp"
    BiggerThan10MP = "isz:lt,islt:10mp"
    BiggerThan12MP = "isz:lt,islt:12mp"
    BiggerThan15MP = "isz:lt,islt:15mp"
    BiggerThan20MP = "isz:lt,islt:20mp"
    BiggerThan40MP = "isz:lt,islt:40mp"
    BiggerThan70MP = "isz:lt,islt:70mp"
    

class AvailableImagesTypes(metaclass=__ABCMeta):
    Face = "itp:face"
    Photo = "itp:photo"
    Clip_art = "itp:clipart"
    Line_drawing = "itp:lineart"
    Animated = "itp:animated"


class AvailableAspectRatios(metaclass=__ABCMeta):
    Tall = "iar:t"
    Square = "iar:s"
    Wide = "iar:w"
    Panoramic = "iar:xw"
    

class AvailableImageFormats(metaclass=__ABCMeta):
    jpg = "ift:jpg"
    gif = "ift:gif"
    png = "ift:png"
    bmp = "ift:bmp"
    svg = "ift:svg"
    webp = "webp"
    ico = "ift:ico"
    