#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
from abc import ABCMeta


class AvailableLanguages(metaclass=ABCMeta):
    Arabic = "lang_ar"
    Armenian = "lang_hy"
    Belorussian = "lang_be"
    Bulgarian = "lang_bg"
    Catalan = "lang_ca"
    Croatian = "lang_hr"
    Czech = "lang_cs"
    Danish = "lang_da"
    Dutch = "lang_nl"
    English = "lang_en"
    Esperanto = "lang_eo"
    Estonian = "lang_et"
    Filipino = "lang_tl"
    Finnish = "lang_fi"
    French = "lang_fr"
    German = "lang_de"
    Greek = "lang_el"
    Hebrew = "lang_iw"
    Hungarian = "lang_hu"
    Icelandic = "lang_is"
    Indonesian = "lang_id"
    Italian = "lang_it"
    Japanese = "lang_ja"
    Korean = "lang_ko"
    Latvian = "lang_lv"
    Lithuanian = "lang_lt"
    Norwegian = "lang_no"
    Persian = "lang_fa"
    Polish = "lang_pl"
    Portuguese = "lang_pt"
    Romanian = "lang_ro"
    Russian = "lang_ru"
    Serbian = "lang_sr"
    Slovak = "lang_sk"
    Slovenian = "lang_sl"
    Spanish = "lang_es"
    Swedish = "lang_sv"
    Thai = "lang_sv"
    Turkish = "lang_tr"
    Ukrainian = "lang_uk"
    Vietnamese = "lang_vi"
    ChineseSimplified = "lang_zh-CN"
    ChineseTraditional = "lang_zh-TW"
