#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
from abc import ABCMeta as __ABCMeta


class InterfaceLanguages(metaclass=__ABCMeta):
    Afrikaans = "af"
    Luo = "ach"
    Akan = "ak"
    Amharic = "am"
    Arabic = "ar"
    Azerbaijani = "az"
    Belorussian = "be"
    Bemba = "bem"
    Bulgarian = "bg"
    Bihari = "bh"
    Bengali = "bn"
    Armenian = "hy"
    Basque = "eu"
    Bork = "xx-bork"
    Bosnian = "bs"
    Cambodian = "km"
    Catalan = "ca"
    Cherokee = "chr"
    Chichewa = "ny"
    ChineseSimplified = "zh-CN"
    ChineseTraditional = "zh-TW"
    Corsican = "co"
    Croatian = "hr"
    Czech = "cs"
    Danish = "da"
    Dutch = "nl"
    ElmerFudd = "xx-elmer"
    English = "en"
    Esperanto = "eo"
    Estonian = "et"
    Ewe = "ee"
    Faroese = "fo"
    Filipino = "tl"
    Finnish = "fi"
    French = "fr"
    Frisian = "fy"
    Ga = "gaa"
    Galician = "gl"
    Georgian = "ka"
    German = "de"
    Greek = "el"
    Guarani = "gn"
    Gujarati = "gu"
    Hacker = "xx-hacker"
    HaitianCreole = "ht"
    Hausa = "ha"
    Hawaiian = "haw"
    Hebrew = "iw"
    Hindi = "hi"
    Hungarian = "hu"
    Icelandic = "is"
    Igbo = "ig"
    Indonesian = "id"
    Interlingua = "ia"
    Irish = "ga"
    Italian = "it"
    Japanese = "ja"
    Javanese = "jw"
    Kannada = "kn"
    Kazakh = "kk"
    Kinyarwanda = "rw"
    Kirundi = "rn"
    Klingon = "xx-klingon"
    Kongo = "kg"
    Korean = "ko"
    KrioSierraLeone = "kri"
    Kurdish = "ku"
    KurdishSorani = "ckb"
    Kyrgyz = "ky"
    Laothian = "lo"
    Latin = "la"
    Latvian = "lv"
    Lingala = "ln"
    Lithuanian = "lt"
    Lozi = "loz"
    Luganda = "lg"
    Macedonian = "mk"
    Malagasy = "mg"
    Malay = "ms"
    Malayalam = "ml"
    Maltese = "mt"
    Maori = "mi"
    Marathi = "mr"
    MauritianCreole = "mfe"
    Moldavian = "mo"
    Mongolian = "mn"
    Montenegrin = "sr-ME"
    Nepali = "ne"
    NigerianPidgin = "pcm"
    NorthernSotho = "nso"
    Norwegian = "no"
    NorwegianNynorsk = "nn"
    Occitan = "oc"
    Oriya = "or"
    Oromo = "ps"
    Pashto = "fa"
    Persian = "fa"
    Pirate = "xx-pirate"
    Polish = "pl"
    PortugueseBrazil = "pt-BR"
    PortuguesePortugal = "pt-PT"
    Punjabi = "pa"
    Quechua = "qu"
    Romanian = "ro"
    Romansh = "rm"
    Runyakitara = "nyn"
    Russian = "ru"
    ScotsGaelic = "gd"
    Serbian = "sr"
    SerboCroatian = "sh"
    Sesotho = "st"
    Setswana = "tn"
    SeychelloisCreole = "crs"
    Shona = "sn"
    Sindhi = "sd"
    Sinhalese = "si"
    Slovak = "sk"
    Slovenian = "sl"
    Somali = "so"
    Spanish = "es"
    SpanishLatinAmerican = "es-419"
    Sundanese = "su"
    Swahili = "sw"
    Swedish = "sv"
    Tajik = "tg"
    Tamil = "ta"
    Tatar = "tt"
    Telugu = "te"
    Thai = "th"
    Tigrinya = "ti"
    Tonga = "to"
    Tshiluba = "lua"
    Tumbuka = "tum"
    Turkish = "tr"
    Turkmen = "tk"
    Twi = "tw"
    Uighur = "ug"
    Ukrainian = "uk"
    Urdu = "ur"
    Uzbek = "uz"
    Vietnamese = "vi"
    Welsh = "cy"
    Wolof = "wo"
    Xhosa = "xh"
    Yiddish = "yi"
    Yoruba = "yo"
    Zulu = "zu"
    Albanian = "sq"
